@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import logging
+import os
 import traceback
 from typing import Tuple, Set
 
@@ -20,11 +21,11 @@ import requests
 
 import magicproxy
 import magicproxy.types
+from . import magictoken
+from . import queries
+from . import scopes
 from .config import Config, load_config
 from .headers import clean_request_headers, clean_response_headers
-from . import magictoken
-from . import scopes
-from . import queries
 from .magictoken import magictoken_params_validate
 
 logger = logging.getLogger(__name__)
@@ -38,29 +39,24 @@ custom_request_headers_to_clean: Set[str] = set()
 
 @app.route("/__magictoken", methods=["POST", "GET"])
 def create_magic_token():
-    CONFIG: Config = app.config["CONFIG"]
-    api_root = CONFIG.api_root
+    config: Config = app.config["CONFIG"]
+    if config is None:
+        return "magic API proxy version " + magicproxy.__version__, 503
+    api_root = config.api_root
     if flask.request.method == "GET":
         return "magic API proxy for " + api_root + " version " + magicproxy.__version__
     params = flask.request.json
     try:
-        magictoken_params_validate(CONFIG, params)
+        magictoken_params_validate(config, params)
     except ValueError as e:
         return str(e), 400
 
-    token = magictoken.create(
-        CONFIG.keys, params["token"], params.get("scopes"), params.get("allowed")
-    )
-
+    token = magictoken.create(config.keys, params["token"], params.get("scopes"), params.get("allowed"))
     return token, 200, {"Content-Type": "application/jwt"}
 
 
-def _proxy_request(
-    request: flask.Request, url: str, headers=None, **kwargs
-) -> Tuple[bytes, int, dict]:
-    clean_headers = clean_request_headers(
-        request.headers, custom_request_headers_to_clean
-    )
+def _proxy_request(request: flask.Request, url: str, headers=None, **kwargs) -> Tuple[bytes, int, dict]:
+    clean_headers = clean_request_headers(request.headers, custom_request_headers_to_clean)
 
     if headers:
         clean_headers.update(headers)
@@ -71,12 +67,7 @@ def _proxy_request(
 
     # Make the API request
     resp = requests.request(
-        url=url,
-        method=request.method,
-        headers=clean_headers,
-        params=dict(request.args),
-        data=request.data,
-        **kwargs,
+        url=url, method=request.method, headers=clean_headers, params=dict(request.args), data=request.data, **kwargs,
     )
 
     response_headers = clean_response_headers(resp.headers)
@@ -89,7 +80,9 @@ def _proxy_request(
 @app.route("/", defaults={"path": ""})
 @app.route("/<path:path>", methods=["POST", "GET", "PATCH", "PUT", "DELETE"])
 def proxy_api(path):
-    CONFIG = app.config["CONFIG"]
+    config = app.config["CONFIG"]
+    if config is None:
+        return "magic API proxy version " + magicproxy.__version__, 503
     auth_token = flask.request.headers.get("Authorization")
     if auth_token is None:
         return "No authorization token presented", 401
@@ -99,14 +92,12 @@ def proxy_api(path):
 
     try:
         # Validate the magic token
-        token_info = magictoken.decode(CONFIG.keys, auth_token)
+        token_info = magictoken.decode(config.keys, auth_token)
     except ValueError:
         return "Not a valid magic token", 400
 
     # Validate scopes against URL and method.
-    if not scopes.validate_request(
-        CONFIG, flask.request.method, path, token_info.scopes, token_info.allowed
-    ):
+    if not scopes.validate_request(config, flask.request.method, path, token_info.scopes, token_info.allowed):
         return (
             "Disallowed by API proxy",
             401,
@@ -115,15 +106,11 @@ def proxy_api(path):
     path = queries.clean_path_queries(query_params_to_clean, path)
 
     response = _proxy_request(
-        request=flask.request,
-        url=f"{CONFIG.api_root}/{path}",
-        headers={"Authorization": f"Bearer {token_info.token}"},
+        request=flask.request, url=f"{config.api_root}/{path}", headers={"Authorization": f"Bearer {token_info.token}"},
     )
 
     try:
-        scopes.response_callback(
-            CONFIG, flask.request.method, path, *response, token_info.scopes
-        )
+        scopes.response_callback(config, flask.request.method, path, *response, token_info.scopes)
     except Exception as e:
         logger.error("exception in response_callback")
         logger.error(e)
@@ -133,11 +120,21 @@ def proxy_api(path):
 
 
 def build_app(config: Config = None):
+    if "COVERAGE_RUN" in os.environ:
+        import coverage
+
+        coverage.process_startup()
     if config is None:
-        config = load_config()
+        try:
+            config = load_config()
+        except RuntimeError:
+            # will run, but in degraded mode (503)
+            pass
     app.config["CONFIG"] = config
     return app
 
 
 def run_app(host, port, config: Config = None):
-    build_app(config).run(host=host, port=port)
+    build_app(config).run(
+        host=host, port=port, use_reloader=os.environ.get("FLASK_USE_RELOADER") is not None,
+    )
